import logging
import logging
import traceback
import uuid
from unittest import mock

from tycho.client import TychoSystem
from tycho.context import Principal, TychoContext

logger = logging.getLogger (__name__)

def __context_start_side_effect (request):    
    """
    This is a test mock. It returns a nominal response for a Tycho app launch (start) request.
    This allows us to test all registry components and their interaction with the Tycho interface
    without a running Tycho API endpoint.
    """
    return TychoSystem (**{
        "status" : "ok",
        "result" : {
            "name"       : request['name'],
            "sid"        : uuid.uuid4 (),
            "containers" :  {
                k : { 'ip_address' : 'x.y.z', 'port-1' : v }
                for k, v in request['services'].items ()
            }
        },
        "message" : "mock: testing..."
    })

def test_context ():
    """ 
    Test the TychoContext. It must be able to 
      * Load the application registry.
      * Demonstrate the ability to scope apps to a product
      * Respond to requests to launch applications which entails
        * Get metadata for each app, formulating a request to Tycho, etc.
    The test reports each failed app launch.
    It does not yet fail the build.
    When we get the initial set of apps completed, failing apps
    will fail the build.

    Example Run:  PYTHONPATH=~/dev/tycho pytest --log-cli-level=INFO test/test_registry.py

    """
    with mock.patch.object (TychoContext, '_start', side_effect=__context_start_side_effect):
        __test_context ()
def __test_context ():
    principal = Principal (username="test_user")
    seen = {}
    failed = []
    successful_total = 0
    failed_total = 0
    for product in [ "braini", "bdc", "scidas", "blackbalsam" ]:
        tc = TychoContext (product=product)
        successful_count = 0
        failed_count = 0
        for app_id, app in tc.apps.items ():
            try:
                if app_id in seen:
                    logger.debug (f"-- skipping seen app {app_id}")
                    continue
                seen [app_id] = app_id
                system = tc.start (principal=principal, app_id=app_id)
                logger.info (f"  -- https://<UX_URL>/private/{app_id}/{principal.username}/{system.identifier}/")
                successful_count = successful_count + 1
            except Exception as e:
                logger.debug (f"App {app_id} failed. {e}")
                traceback.print_exc ()
                failed.append (app_id)
                failed_count = failed_count + 1
        logger.info (f"{product.upper()} had {successful_count} successful apps and {failed_count} failed apps.")
        successful_total = successful_total + successful_count
        failed_total = failed_total + failed_count
    logger.info (f"total of {successful_total} successful apps and {failed_total} failed apps.")
    failed_copy = failed.copy ()
    failed_copy.insert (0, "")
    fail_list = "\n==    * ".join (failed_copy)
    if len(failed) > 0:
        logger.error (f"""
=======================================
== Failed to launch the following apps:{fail_list}
=======================================""")



from pytest import mark

from tycho.client import TychoClient


@mark.skip(reason="Need connection to a running Kubernetes cluster and a valid guid.")
@mark.kubernetes
class TestModify:

    def test_modify_functional(self, modify_data, mock_modify_function_env_variables):
        client = TychoClient()
        response = client.patch(modify_data)
        if response["status"] == "success":
            assert True
        else:
            assert False



from unittest import mock
import pytest
from tycho.tycho_utils import Resource
import os

class TestResource:

    testdata = [
        ("test.yaml", None, "yaml"),
        ("test.yaml", 'yaml', "yaml"),
        ("test.json", None, "json"),
        ("test.json", 'json', "json"),
        ("test", 'yaml', "yaml"),
        ("test.exe", None, "json"),
    ]

    @pytest.mark.parametrize("resource_name,format,correct_type", testdata)
    def test_get_resource_obj(self, resource_name, format, correct_type, mocker):
        # Set up mocks
        mocker.patch('tycho.tycho_utils.Resource.get_resource_path', return_value="")
        mocker.patch('os.path.exists', return_value=True)
        mocker.patch('tycho.tycho_utils.Resource.load_json', return_value="json")
        mocker.patch('tycho.tycho_utils.Resource.load_yaml', return_value="yaml")

        # Test get_resource_obj
        resource = Resource()
        result = resource.get_resource_obj(resource_name, format)
        assert result == correct_type



import os

import yaml

from tycho.config import Config
from tycho.model import System


def test_system_model(request):
    """ Test that the abstract model parses structures as we expect and puts the pieces
    where they belong. """
    print(f"{request.node.name}")
    system = System(**{
        "config": Config(),
        "name": "test",
        "identifier":System.get_identifier(),
        "principal": {"username": "renci"},
        "service_account": "default",
        "conn_string": "",
        "proxy_rewrite_rule": False,
        "containers": [
            {
                "name": "nginx-container",
                "image": "nginx:1.9.1",
                "limits": [{
                    "cpus": "0.5",
                    "memory": "512M"
                }],
                "volumes": [],
            }
        ]
    })
    print(system.containers[0].limits)
    assert system.name.startswith('test-')
    assert system.containers[0].name == 'nginx-container'
    assert system.containers[0].image == 'nginx:1.9.1'
    assert system.containers[0].limits['cpus'] == "0.5"
    assert system.containers[0].limits['memory'] == "512M"


def test_system_parser(request):
    """ Test parsing of a docker-compose into the standard model. """
    print(f"{request.node.name}")
    base_dir = os.path.dirname(os.path.dirname(__file__))
    spec_path = os.path.join(base_dir, "tycho", "sample", "jupyter-ds", "docker-compose.yaml")
    with open(spec_path, "r") as stream:
        structure = yaml.load(stream, Loader=yaml.FullLoader)
        system = System.parse(
            config=Config(),
            name="jupyter-ds",
            principal='{"username": "renci"}',
            system=structure,
            service_account="default")

        print(f"{system}")
        assert system.name.startswith('jupyter-ds')
        assert system.containers[0].name == 'jupyter-ds'
        assert system.containers[0].image == 'jupyter/datascience-notebook'
        assert system.containers[0].limits.cpus == '0.01'
        assert system.containers[0].limits.memory == '50M'
        assert system.containers[0].requests.cpus == '0.01'
        assert system.containers[0].requests.memory == '20M'
        assert system.containers[0].ports[0]['containerPort'] == '8888'
        assert system.containers[0].volumes[0] == 'pvc://cloud-top/projects:/work/data'
